"""
Utils.

Re-usable methods and functions for all test cases.
"""

import random
import string


def generate_random_string(length: int) -> str:
    """Generate a random string of a given length"""
    return "".join(random.choices(string.ascii_lowercase + string.digits, k=length))


def generate_random_email(domain: str = "yopmail.net", length: int = 9) -> str:
    """Get a random email to be using in test cases."""
    random_string = "".join(random.choice(string.ascii_lowercase) for _ in range(length))
    return "trellis-test." + random_string + "@" + domain



# mypy: ignore-errors
# pylint: skip-file

from typing import Optional, get_args, get_origin

from beanie.odm.fields import LinkInfo, LinkTypes
from beanie.odm.registry import DocsRegistry
from beanie.odm.utils.init import Initializer as Initializer
from pydantic.fields import FieldInfo
from pydantic.typing import get_origin

from .link import Link as MLink

original__detect_link = Initializer.detect_link


def detect_link(self, field: FieldInfo, field_name: str) -> Optional[LinkInfo]:
    """
    It detects link and returns LinkInfo if any found.

    :param field: ModelField
    :return: Optional[LinkInfo]
    """
    if get_origin(field.annotation) is MLink:
        args = get_args(field.annotation)
        return LinkInfo(
            field_name=field_name,
            lookup_field_name=field_name,
            document_class=DocsRegistry.evaluate_fr(args[0]),  # type: ignore
            link_type=LinkTypes.DIRECT,
        )
    return original__detect_link(self, field=field, field_name=field_name)


Initializer.detect_link = detect_link
